@@ -18,10 +18,10 @@ from google.cloud import dataproc_v1, storage
 
 import bigflow
 import bigflow.configuration
-import bigflow.resources
 import bigflow.commons
 import bigflow.build.reflect
 import bigflow.build.pip
+from bigflow.commons import public
 
 from bigflow.workflow import DEFAULT_EXECUTION_TIMEOUT_IN_SECONDS
 from bigflow._version import __version__ as bf_version
@@ -33,6 +33,7 @@ __all__ = [
 ]
 
 
+@public()
 class PySparkJob(bigflow.Job):
 
     def __init__(
@@ -49,7 +50,8 @@ class PySparkJob(bigflow.Job):
         worker_machine_type: str = 'n1-standard-1',
         env: typing.Optional[str] = None,
         project_name: typing.Optional[str] = None,
-        execution_timeout_sec: int = DEFAULT_EXECUTION_TIMEOUT_IN_SECONDS
+        execution_timeout_sec: int = DEFAULT_EXECUTION_TIMEOUT_IN_SECONDS,
+        internal_ip_only: bool = False
     ):
         self.id = id
 
@@ -70,13 +72,13 @@ class PySparkJob(bigflow.Job):
         self.worker_num_instances = worker_num_instances
 
         if isinstance(pip_packages, pathlib.Path):
-            self.pip_packages = bigflow.resources.read_requirements(pip_packages)
+            self.pip_packages = bigflow.build.pip.read_requirements(pip_packages)
         else:
             self.pip_packages = pip_packages
 
-        self._project = project_name or bigflow.build.reflect.infer_project_name(stack=2)
-
+        self._project_pkg_path = project_name or bigflow.build.reflect.locate_project_path(project_name)
         self.execution_timeout_sec = execution_timeout_sec
+        self.internal_ip_only = internal_ip_only
 
     def _generate_internal_jobid(self, context):
         job_random_suffix = ''.join(random.choices(string.ascii_lowercase + string.digits, k=10))
@@ -111,6 +113,7 @@ class PySparkJob(bigflow.Job):
                 requirements=self.pip_packages,
                 worker_machine_type=self.worker_machine_type,
                 worker_num_instances=self.worker_num_instances,
+                internal_ip_only=self.internal_ip_only
             )
             yield cluster_name
         finally:
@@ -150,7 +153,7 @@ class PySparkJob(bigflow.Job):
         logger.info("Prapare and upload python package...")
         bucket = storage_client.get_bucket(self.bucket_id)
 
-        egg_local_path = str(bigflow.build.reflect.build_egg(self._project))
+        egg_local_path = str(bigflow.build.reflect.build_egg(self._project_pkg_path))
         egg_path = _upload_egg(egg_local_path, bucket, job_internal_id)
 
         driver_path = f"{job_internal_id}/{self.driver_filename}"
@@ -181,17 +184,18 @@ def generate_driver_script(callable, env):
     env = dict(env or {})
     return textwrap.dedent(f"""
         # Generated by `bigflow.dataproc.generate_and_upload_driver`
-
-        import os
-        import base64
-        import pickle
+        import os, base64, pickle
 
         # overwrite env variables
         os.environ.update({env!r})
 
         # triggers configuration of logging
-        import bigflow
-        bigflow._maybe_init_logging_from_env()
+        try:
+            from bigflow.log import maybe_init_logging_from_env
+        except ImportError:
+            pass
+        else:
+            maybe_init_logging_from_env()
 
         # unpickle'n'call {callable!r}
         data = {base64.b85encode(pickled)!r}
@@ -240,7 +244,15 @@ def _print_job_output_log(storage_client, dataproc_cluster_client, project_id, r
     logger.info("JOB OUTPUT:\n=====\n%s\n=====", log_buffer.getvalue().decode())
 
 
-def _create_cluster(dataproc_cluster_client, project_id, region, cluster_name, requirements, worker_num_instances, worker_machine_type):
+def _create_cluster(
+        dataproc_cluster_client,
+        project_id,
+        region,
+        cluster_name,
+        requirements,
+        worker_num_instances,
+        worker_machine_type,
+        internal_ip_only: bool = False):
     packages = " ".join(filter(None, requirements))
     cluster_data = {
         # "project_id": project_id,
@@ -263,7 +275,8 @@ def _create_cluster(dataproc_cluster_client, project_id, region, cluster_name, r
                 {"executable_file": "gs://goog-dataproc-initialization-actions-{}/python/pip-install.sh".format(region)}
             ],
             "gce_cluster_config": {
-                "metadata": [('PIP_PACKAGES', packages)]
+                "metadata": [('PIP_PACKAGES', packages)],
+                "internal_ip_only": internal_ip_only
             }
         },
     }
