@@ -1,15 +1,21 @@
+import datetime
 from unittest import TestCase
 import mock
 
-from bigflow.interactive import InteractiveDatasetManager
-from bigflow.interactive import interactive_component
-from bigflow.job import DEFAULT_RETRY_COUNT
-from bigflow.job import DEFAULT_RETRY_PAUSE_SEC
+from google.api_core.exceptions import BadRequest
+
+import bigflow
+
+from bigflow.bigquery.interactive import InteractiveDatasetManager
+from bigflow.bigquery.interactive import interactive_component
+from bigflow.bigquery.job import DEFAULT_RETRY_COUNT
+from bigflow.bigquery.job import DEFAULT_RETRY_PAUSE_SEC
+from bigflow.bigquery.interactive import log_syntax_error
 
 
 class OperationLevelDatasetManagerTestCase(TestCase):
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_preserve_dataset_order(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, {'dependency_kwargs': kwargs})
@@ -33,9 +39,9 @@ class OperationLevelDatasetManagerTestCase(TestCase):
         job = standard_component.to_job()
 
         # then
-        job.run('2019-01-01')
+        job.execute(bigflow.JobContext.make())
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_pass_all_arguments_to_core_dataset_manager_on_run(self, create_dataset_manager_mock):
         # given
         fake_dataset_manager = mock.Mock()
@@ -58,7 +64,7 @@ class OperationLevelDatasetManagerTestCase(TestCase):
 
         # when
         job = standard_component.to_job()
-        job.run('2019-01-01')
+        job.execute(bigflow.JobContext.make())
 
         # then
         fake_dataset_manager.assert_has_calls([
@@ -77,7 +83,7 @@ class OperationLevelDatasetManagerTestCase(TestCase):
                 table_name='some table', df='some dataframe', partitioned=False, custom_run_datetime='2019-12-12')
         ])
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_run_specified_operation(self, create_dataset_manager_mock):
         # given
         dataset_manager_mock = mock.Mock()
@@ -104,7 +110,7 @@ class OperationLevelDatasetManagerTestCase(TestCase):
             custom_run_datetime=None,
             partitioned=True)])
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_peek_operation_results(self, create_dataset_manager_mock):
         # given
         dataset_manager_mock = mock.Mock()
@@ -142,7 +148,7 @@ class OperationLevelDatasetManagerTestCase(TestCase):
             'collect sql 5\nLIMIT 1000')
 
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_allow_setting_peek_limit(self, create_dataset_manager_mock):
         # given
         dataset_manager_mock = mock.Mock()
@@ -169,7 +175,7 @@ class OperationLevelDatasetManagerTestCase(TestCase):
 
 class InteractiveComponentToJobTestCase(TestCase):
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_pass_configuration_to_job(self, create_dataset_manager_mock):
         # given
 
@@ -190,7 +196,7 @@ class InteractiveComponentToJobTestCase(TestCase):
         @interactive_component(ds=default_dataset)
         def standard_component(ds):
             # then
-            self.assertEqual(ds.dt, '2019-01-01')
+            self.assertEqual(ds.dt, '2019-01-01 00:00:00')
             self.assertEqual(ds.extras, {'extra': 'param'})
             self.assertEqual(ds.client, 'client')
 
@@ -202,9 +208,9 @@ class InteractiveComponentToJobTestCase(TestCase):
         self.assertEqual(job.retry_count, DEFAULT_RETRY_COUNT)
         self.assertEqual(job.retry_pause_sec, DEFAULT_RETRY_PAUSE_SEC)
         self.assertEqual(job.dependency_configuration, {'ds': default_dataset.config})
-        job.run('2019-01-01')
+        job.execute(bigflow.JobContext.make(runtime=datetime.datetime(2019, 1, 1)))
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_turn_component_to_job_with_redefined_dependencies(self, create_dataset_manager_mock):
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, {'dependency_kwargs': kwargs})
 
@@ -242,9 +248,9 @@ class InteractiveComponentToJobTestCase(TestCase):
         job = standard_component.to_job(dependencies_override={'ds2': modified_dataset})
 
         # then
-        job.run('2019-01-01')
+        job.execute(bigflow.JobContext.make())
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_not_mutate_dependency_configuration_when_redefining_dependencies(self, create_dataset_manager_mock):
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, {'dependency_kwargs': kwargs})
 
@@ -265,9 +271,9 @@ class InteractiveComponentToJobTestCase(TestCase):
 
         # when
         standard_component.to_job(dependencies_override={'ds': modified_dataset})
-        standard_component.to_job().run('2019-01-01')
+        standard_component.to_job().execute(bigflow.JobContext.make())
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_wrap_each_dependency_into_operation_level_dataset_manager(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, {'dependency_kwargs': kwargs})
@@ -281,12 +287,12 @@ class InteractiveComponentToJobTestCase(TestCase):
             self.assertEqual(ds1._peek_limit, None)
 
         # when
-        fake_component.to_job().run('2019-01-01')
+        fake_component.to_job().execute(bigflow.JobContext.make(runtime_str='2019-01-01'))
 
 
 class InteractiveComponentRunTestCase(TestCase):
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_wrap_each_dependency_into_operation_level_dataset_manager(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, {'dependency_kwargs': kwargs})
@@ -312,7 +318,7 @@ class InteractiveComponentRunTestCase(TestCase):
         # when 2
         fake_component.run('2019-01-01', operation_name='some_operation')
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_pass_configuration_to_job(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, {'dependency_kwargs': kwargs})
@@ -332,7 +338,7 @@ class InteractiveComponentRunTestCase(TestCase):
 
 class InteractiveComponentPeekTestCase(TestCase):
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_wrap_each_dependency_into_operation_level_dataset_manager(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, mock.MagicMock())
@@ -349,7 +355,7 @@ class InteractiveComponentPeekTestCase(TestCase):
         # when
         fake_component.peek('2019-01-01', operation_name='some_operation', limit=100)
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_pass_configuration_to_job(self, create_dataset_manager_mock):
         # given
         def fake_dataset_manager(kwargs):
@@ -374,7 +380,7 @@ class InteractiveComponentPeekTestCase(TestCase):
         # when
         fake_component.peek('2019-01-01', operation_name='some_operation')
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_return_peeked_operation_result(self, create_dataset_manager_mock):
         # given
         def fake_dataset_manager(kwargs):
@@ -397,7 +403,7 @@ class InteractiveComponentPeekTestCase(TestCase):
         # then
         self.assertEqual(peek_result, 'peeked_value')
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_not_allow_peeking_with_none_argument(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, mock.Mock())
@@ -422,7 +428,7 @@ class InteractiveComponentPeekTestCase(TestCase):
             # when
             fake_component.peek(None)
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_throw_error_when_operation_not_found(self, create_dataset_manager_mock):
         # given
         create_dataset_manager_mock.side_effect = lambda **kwargs: (None, mock.Mock())
@@ -440,7 +446,7 @@ class InteractiveComponentPeekTestCase(TestCase):
 
 class InteractiveDatasetManagerTestCase(TestCase):
 
-    @mock.patch('bigflow.job.create_dataset_manager')
+    @mock.patch('bigflow.bigquery.job.create_dataset_manager')
     def test_should_create_component_for_specified_operation(self, create_dataset_manager_mock):
         # given
         dataset_manager_mock = mock.Mock()
@@ -451,16 +457,24 @@ class InteractiveDatasetManagerTestCase(TestCase):
         write_append_component = dataset.write_append('table', 'some sql')
         write_tmp_component = dataset.write_tmp('table', 'some sql')
         collect_component = dataset.collect('some sql')
+        collect_list_component = dataset.collect_list('some sql', True)
         dry_run_component = dataset.dry_run('some sql')
         load_table_from_dataframe_component = dataset.load_table_from_dataframe('table', 'df')
+        create_table_from_schema_component = dataset.create_table_from_schema('table', [{}])
+        insert_component = dataset.insert('table', [{}])
+        delete_dataset_component = dataset.delete_dataset()
 
         # when
         write_truncate_component.run()
         write_append_component.run()
         write_tmp_component.run()
         collect_component.run()
+        collect_list_component.run()
         dry_run_component.run()
         load_table_from_dataframe_component.run()
+        create_table_from_schema_component.run()
+        insert_component.run()
+        delete_dataset_component.run()
 
         # then
         dataset_manager_mock.assert_has_calls([
@@ -481,6 +495,10 @@ class InteractiveDatasetManagerTestCase(TestCase):
             mock.call.collect(
                 sql='some sql',
                 custom_run_datetime=None),
+            mock.call.collect_list(
+                sql='some sql',
+                custom_run_datetime=None,
+                record_as_dict=True),
             mock.call.dry_run(
                 sql='some sql',
                 custom_run_datetime=None),
@@ -489,4 +507,45 @@ class InteractiveDatasetManagerTestCase(TestCase):
                 df='df',
                 custom_run_datetime=None,
                 partitioned=True),
+            mock.call.create_table_from_schema(
+                table_name='table',
+                schema=[{}],
+                table=None),
+            mock.call.insert(
+                table_name='table',
+                records=[{}],
+                partitioned=True),
+            mock.call.remove_dataset(),
         ])
+
+
+def raise_import_error():
+    raise ImportError()
+
+
+PARAMETER1 = '1'
+PARAMETER2 = '2'
+
+
+class LogSyntaxErrorTestCase(TestCase):
+
+    @mock.patch('bigflow.bigquery.interactive.logger')
+    def test_should_catch_bad_request_and_log_message(self, logger_mock: mock.Mock):
+        # when
+        self.bad_request_occurred()
+
+        # then
+        logger_mock.error.assert_called_once_with('Syntax error')
+
+    def test_should_pass_all_parameters(self):
+        # expect
+        self._test_should_pass_all_parameters(PARAMETER1, PARAMETER2)
+
+    @log_syntax_error
+    def _test_should_pass_all_parameters(self, par1, par2):
+        self.assertEqual(par1, PARAMETER1)
+        self.assertEqual(par2, PARAMETER2)
+
+    @log_syntax_error
+    def bad_request_occurred(self):
+        raise BadRequest('Syntax error')
\ No newline at end of file
